package jadex.adapter.jade;

import jade.core.AID;
import jade.lang.acl.ACLMessage;
import jade.content.onto.basic.Action;
import jadex.adapter.IJadexAgent;
import jadex.adapter.fipa.FipaMessageAdapter;
import jadex.adapter.fipa.SFipa;
import jadex.runtime.impl.RCapability;

import java.util.Iterator;

/**
 *  Message adapter for Jade messages.
 */
public class JadeMessageAdapter extends FipaMessageAdapter
{
	//-------- attributes --------

	/** The message content. */
	//private Object content;

	/** The Jade agent is used to access the content manager. */
	//private Agent	jadeagent;

	/** The Jadex agent is used to access the logger. */
	private IJadexAgent	jadexagent;

	/** The decoded content (cached for speed). */
	// Hack!!! implement this generically?
	protected Object	content;

	//-------- constructors --------

	/**
	 *  Create a new message adapter.
	 */
	//public JadeMessageAdapter(ACLMessage message, Agent jadeagent, IJadexAgent agent)
	public JadeMessageAdapter(ACLMessage message, IJadexAgent agent)
	{
		super(agent, message);
		//this.jadeagent = jadeagent;
		this.jadexagent	= agent;
	}

	//-------- methods --------

	/**
	 *  Get the value for a parameter, or the values
	 *  for a parameter set.
	 *  Parameter set values can be provided as array, collection,
	 *  iterator or enumeration.
	 */
	public Object	getRawValue(String name)
	{
		Object ret = null;
		ACLMessage message = (ACLMessage)getMessage();

		if(name.equals(SFipa.ENCODING))
		{
			ret = message.getEncoding();
		}
		else if(name.equals(SFipa.IN_REPLY_TO))
		{
			ret = message.getInReplyTo();
		}
		else if(name.equals(SFipa.LANGUAGE))
		{
			ret = message.getLanguage();
		}
		else if(name.equals(SFipa.ONTOLOGY))
		{
			ret = message.getOntology();
		}
		else if(name.equals(SFipa.PROTOCOL))
		{
			ret = message.getProtocol();
		}
		else if(name.equals(SFipa.REPLY_BY))
		{
			ret = message.getReplyByDate();
		}
		else if(name.equals(SFipa.REPLY_WITH))
		{
			ret = message.getReplyWith();
		}
		else if(name.equals(SFipa.CONVERSATION_ID))
		{
			ret = message.getConversationId();
		}
		else if(name.equals(SFipa.PERFORMATIVE))
		{
			ret = ACLMessage.getPerformative(message.getPerformative()).toLowerCase();
		}
		else if(name.equals(SFipa.CONTENT))
		{
			ret = message.getContent();
		}
		else if(name.equals(SFipa.SENDER))
		{
			ret = SJade.convertAIDtoFipa(message.getSender());
		}
		else if(name.equals(SFipa.REPLY_TO))
		{
			Iterator	it	= message.getAllReplyTo();
			if(it.hasNext())
				ret	= SJade.convertAIDtoFipa((AID)it.next());
			if(it.hasNext())
				jadexagent.getLogger().warning("Ignoring additional reply tos of message: "+message);
		}
		else if(name.equals(SFipa.RECEIVERS))
		{
			final Iterator	it	= message.getAllReplyTo();
			if(it.hasNext())
			{
				ret	= new Iterator()
				{
					public boolean hasNext()
					{
						return it.hasNext();
					}
					public Object next()
					{
						return SJade.convertAIDtoFipa((AID)it.next());
					}
					public void remove()
					{
						it.remove();
					}
				};
			}
		}

		return ret;
	}

	/**
	 *  Get the value for a parameter, or the values
	 *  for a parameter set from a native message.
	 *  Parameter set values can be provided as array, collection,
	 *  iterator or enumeration.
	 */
	public Object	getValue(String name, RCapability scope)
	{
		Object ret = super.getValue(name, scope);

		// Treat action-class as parameter of jade message.
		if(name.equals(SFipa.ACTION_CLASS) && ret instanceof Action)
			ret = ((Action)ret).getAction().getClass();

		return ret;
	}

	/**
	 *  Get the value for a parameter, or the values
	 *  for a parameter set from a native message.
	 *  Parameter set values can be provided as array, collection,
	 *  iterator or enumeration.
	 * /
	public Object	getValue(String name)
	{
		Object val;
		// Todo: implement generically for all message types.
		if(SFipa.CONTENT.equals(name))
		{
			if(content!=null)
			{
				val	= content;
			}
			else if((val=getRawValue(name)) != null)
			{
				String lang = (String)getValue(SFipa.LANGUAGE);
				String onto = (String)getValue(SFipa.ONTOLOGY);
				if(lang!=null && onto!=null)
				{
					try
					{
						Properties props = new Properties();
						props.setProperty(SFipa.LANGUAGE, lang);
						props.setProperty(SFipa.ONTOLOGY, onto);

						IContentCodec codec = agent.getContentCodec(props);
						val = codec.decode((String)val);
					}
					catch(Exception e)
					{
						try
						{
							val = jadeagent.getContentManager().extractContent((ACLMessage)getMessage());
						}
						catch(Exception e2)
						{
							val = new ContentException(e2.getMessage()+"\n"+getMessage(), e2);
						}
					}
					content	= val;
				}
			}
		}
		else
		{
			val = getRawValue(name);
		}

		return val;
	}*/

	/**
	 *  Get the string representation.
	 */
	public String toString()
	{
		return "JadeMessageAdapter: "+getMessage();
	}

	/** The X_MESSAGE_ID. */
	static final String	MESSAGE_ID	= "MessageID";
	
	/** 
	 *  Get the id.
	 *  @return The message id.
	 */
	public String getId()
	{
		ACLMessage message = (ACLMessage)getMessage();
		return message.getUserDefinedParameter(MESSAGE_ID);
	}
	
	/** 
	 *  Set the id.
	 *  @param id
	 */
	public void setId(String id) 
	{
		ACLMessage message = (ACLMessage)getMessage();
		message.addUserDefinedParameter(MESSAGE_ID, id);
	}
}
