package	jadex.adapter.jade.planlib;

import jade.content.onto.basic.Action;
import jade.content.onto.basic.Result;
import jade.domain.FIPAAgentManagement.*;
import jadex.adapter.fipa.AgentDescription;
import jadex.adapter.jade.SJade;
import jadex.runtime.IGoal;


/**
 *  The df search plan searches a df for agents
 *  matching a df agent description template.
 *  The parameters have to be specified in the goal.
 *  The result of the deregister action is stored in the goal.
 */
public class JadeDFSearchPlan extends JadeDFPlan
{
	//-------- attributes --------

	/** The search constraints (defaults to empty constraints). */
	protected SearchConstraints	constraints;

	//-------- constructors --------

	/**
	 *  Create a new df search plan.
	 *  The parameters have to be specified in the goal.
	 */
	public JadeDFSearchPlan()
	{
		Object constr = getParameter("constraints").getValue();
		if(constr!=null)
		{
			this.constraints = SJade.convertSearchConstraintstoJade((jadex.adapter.fipa.SearchConstraints)constr);
		}
		else
		{
			this.constraints = new SearchConstraints(); // needed???
		}
	}

	//-------- template methods --------

	/**
	 *  The body.
	 */
	public void	body()
	{
		// Create and dispatch search goal.
		Search	search	= new Search();
		search.setDescription(getAgentDescription());
		search.setConstraints(constraints);
		Action action = new Action();
		action.setActor(SJade.convertAIDtoJade(getDF()));
		action.setAction(search);

		IGoal	req	= createGoal("rp_initiate");
		req.getParameter("receiver").setValue(getDF());
		req.getParameter("action").setValue(action);
		req.getParameter("ontology").setValue(FIPAManagementOntology.NAME);
		req.getParameter("language").setValue(jade.domain.FIPANames.ContentLanguage.FIPA_SL0);
		dispatchSubgoalAndWait(req);

		Result res = (Result)req.getParameter("result").getValue();
		Object[]	items	= res.getItems().toArray();

		AgentDescription[]	descriptions	= new AgentDescription[items.length];
		for(int i=0; i<items.length; i++)
		{
			descriptions[i]	= SJade.convertAgentDescriptiontoFipa((DFAgentDescription)items[i]);
		}
		getParameterSet("result").addValues(descriptions);

		//System.out.println("Search finished succ: "+descriptions);
	}
}

