package jadex.adapter.jade.tools.logger.gui;


import jade.core.AID;

import javax.swing.*;
import java.awt.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;


/**
 * The List for the agents on the Agent Canvas. Implements Serializable for saving
 * data to the binary snapshot file.
 */


public class AgentList extends JPanel implements Serializable
{

    protected JList list;
    protected DefaultListModel listModel;
    protected ListCellRenderer listRenderer;

    /**
     * Default constructor for the class <em>AgentList</em>
     */
    public AgentList(Font font)
    {
        list = new JList();
        list.setFont(font);

        listModel = new DefaultListModel();
        listRenderer = new AgentCellRenderer();

        list.setModel(listModel);
        list.setCellRenderer(listRenderer);
    }

    /**
     * Add an agent to the list.
     *
     * @param agent AID of the agent to add
     */
    public void addAgent(AID agent)
    {
//        agents.add(agent);
        if (!listModel.contains(agent))
        {
            listModel.addElement(agent);
        }
    }

    /**
     * Removes an agent from the list
     *
     * @param agent AID of the agent to remove
     */
    public void removeAgent(AID agent)
    {
//        agents.remove(agent);
        listModel.removeElement(agent);

    }

    /**
     * Clears the agent list
     */
    public void removeAllAgents()
    {
//        agents.clear();
        listModel.removeAllElements();
    }

    /**
     * Verifies if an agent is present on the canvas
     *
     * @param agent name of the agent to check for
     */
    public boolean isPresent(AID agent)
    {
        Enumeration lme = listModel.elements();
        while (lme.hasMoreElements())
        {
            AID a = (AID)lme.nextElement();
            if (a.equals(agent))
            {
                return true;
            }

        }
        return false;
    }


    public ArrayList getAgentList()
    {
        ArrayList agentlist = new ArrayList(listModel.getSize());

        Enumeration lme = listModel.elements();
        while (lme.hasMoreElements())
        {
            AID agent = (AID) lme.nextElement();
            agentlist.add(agent);

        }
        return agentlist;

    }

    class AgentCellRenderer extends JLabel implements ListCellRenderer
    {
        public Component getListCellRendererComponent(
                JList list,
                Object value,
                int index,
                boolean isSelected,
                boolean cellHasFocus)
        {
            AID agent = (AID) value;
            setText(agent.getName());
            if (isSelected)
            {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            }
            else
            {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            setEnabled(list.isEnabled());
            setFont(list.getFont());
            setOpaque(true);
            return this;
        }
    }


}  // End class AgentList
