package jadex.adapter.jade.tools.logger.jadeexample;

import java.util.logging.*;
import jade.core.Agent;

/**
 *  The log agent.
 */
public class LogAgent extends Agent
{
	//-------- constants --------

    protected static final long INCREMENT_TIME = 3000; // 3sec.
    protected static final long RESET_TIME = 10000; // 10sec.

	//-------- attributes --------

	/** Some data. */
    protected int data = 0;

	/** The logger. */
    private Logger logger;

	//-------- constructors --------

	/**
	 *  Called when agent is born.
	 */
    protected void setup()
    {
        // Add increment and reset behaviour
        addBehaviour(new BeliefIncrementBehaviour(this));
        addBehaviour(new BeliefResetBehaviour(this));
    }

	//-------- methods --------

    /**
     *  Get the logger for this agent.
     *  @return The logger.
     */
    public Logger getLogger()
    {
        // If logger does not already exists, create it
        if (logger == null)
        {
            // get logger with agent's name
            String name = getName();
            logger = Logger.getLogger(name);

            // set logging properties
            // the level of the logger
            logger.setLevel(Level.ALL);
            // if logger should use Handlers of parent (global) logger
            // the global logger has a ConsoleHandler(Level:INFO) by default
            logger.setUseParentHandlers(false);

        }
        return logger;
    }
}
