/*****************************************************************
 JADE - Java Agent DEvelopment Framework is a framework to develop
 multi-agent systems in compliance with the FIPA specifications.
 Copyright (C) 2000 CSELT S.p.A.

 GNU Lesser General Public License

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation,
 version 2.1 of the License.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 *****************************************************************/


package jadex.adapter.jade.tools.rma;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.List;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;
import jadex.adapter.jade.JadeAgentAdapter;
import jadex.adapter.jade.tools.common.*;
import jadex.config.Configuration;
import jadex.model.*;
import jadex.util.*;

/**
 * @author Original Jade version by: Francisco Regi, Andrea Soracchi - Universita` di Parma
 * @version $Date: 2007-02-13 10:52:08 +0000 (Tue, 13 Feb 2007) $ $Revision: 4839 $
 */
class StartDialog extends JDialog implements ActionListener
{
	protected static int OK_BUTTON = 0;
	protected static int CANCEL_BUTTON = 1;

	protected static final String modeltooltip = "ADF file to load an agent model from";
	protected static final String configtooltip = "Starting configuration for the agent";
	protected static final String argagenttooltip = "Arguments passed to the agent constructor";
	protected static final String agentnametooltip = "Name of the Agent to start";
	protected static final String classnametooltip = "Class Name of the Agent to start";
	protected static final String containertooltip = "Container on which the Agent will start";

	protected static Map	loadedmodels	= new HashMap();

	//-------- object variables --------

	protected IMElement model;
	protected boolean detailson;
	protected int choice;

	protected JComboBox modelname;
	protected JFileChooser modelchooser;
	protected JComboBox configchooser;
	protected JTextField arguments;
	protected JTextField agentname;
	protected JTextField classname;
	protected JTextField container;
	protected JButton okbutton;
	protected JButton cancelbutton;
	protected JButton resetbutton;
	protected JButton reloadbutton;
	protected JButton detailsbutton;
	protected ElementPanel modeldesc;

	protected Properties props;
	protected ChangeListener	detailslistener;
	protected Dimension	smallsize;	// Size when description is closed.
	protected Dimension largesize;	// Size when description is open.
	protected String	lastfile;	// The last selected file (for reload).
	
	
	//-------- constructor --------

	/**
	 *  Create a new Start dialog.
	 *  @param frame The frame.
	 */
	protected StartDialog(Frame frame)
	{
		super(frame, "Insert Start Parameters", true);

		// Create gui elements

		//modeldesc.setEditable(false);
		modelname = new JComboBox();
		modelchooser = new JFileChooser(".");
		modelchooser.setAcceptAllFileFilterUsed(true);
		FileFilter filter = new FileFilter()
		{
			public String getDescription()
			{
				return "ADFs";
			}

			public boolean accept(File f)
			{
				String name = f.getName();
				//return f.isDirectory() || name.endsWith(".properties") || name.endsWith(".xml") ||
				//		(name.endsWith(".class") && name.indexOf("Model")!=-1);
				return f.isDirectory() ||  name.endsWith(".xml") ||  name.endsWith(".jar");
			}
		};

		okbutton = new JButton("OK");
		cancelbutton = new JButton("Cancel");
		resetbutton = new JButton("Reset");
		reloadbutton = new JButton("Reload");
		detailsbutton = new JButton("Description -->");

		modelchooser.addChoosableFileFilter(filter);
		modelchooser.setFileFilter(filter);
		modelchooser.setDialogType(JFileChooser.OPEN_DIALOG);
		modelname.setEditable(true);
		modelname.setToolTipText(modeltooltip);
		JLabel modellabel = new JLabel("Model (ADF):");
		modellabel.setToolTipText(modeltooltip);

		configchooser = new JComboBox();
		configchooser.setEditable(true);
		configchooser.setToolTipText(argagenttooltip);
		JLabel configurationlabel = new JLabel("Configuration:");
		configchooser.setToolTipText(configtooltip);

		arguments = new JTextField();
		arguments.setEditable(true);
		arguments.setToolTipText(argagenttooltip);
		JLabel argumentslabel = new JLabel("Arguments:");
		argumentslabel.setToolTipText(argagenttooltip);

		agentname = new JTextField();
		agentname.setToolTipText(agentnametooltip);
		JLabel agentnamelabel = new JLabel("Agent Name:");
		agentnamelabel.setToolTipText(agentnametooltip);

		classname = new JTextField("jade.core.Agent");
		classname.setEditable(true);
		classname.setToolTipText(classnametooltip);
		JLabel classnamelabel = new JLabel("Class Name:");
		classnamelabel.setToolTipText(classnametooltip);

		container = new JTextField("");
		container.setToolTipText(containertooltip);
		JLabel containerlabel = new JLabel("Container:");
		containerlabel.setToolTipText(containertooltip);
		container.setEditable(false);

		// The production mode flag.
		/*final JCheckBox prodmode = new JCheckBox("Development mode");
		if(SXML.getFactory().equals(SXML.FACTORY_JBIND))
			prodmode.setSelected(true);
		else
			prodmode.setSelected(false);
		prodmode.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(prodmode.isSelected())
					SXML.setFactory(SXML.FACTORY_JBIND);
				else
					SXML.setFactory(SXML.FACTORY_JIBX);
			}
		});*/

		
		modeldesc = new ElementPanel("Description", null);

		JPanel n2 = new JPanel(new BorderLayout());
		JPanel north = new JPanel(new GridBagLayout());
		north.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Choose an Agent"));
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add("North", n2);
		Insets insets = new Insets(1, 3, 1, 3);

		north.add(modellabel, new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.CENTER,
			GridBagConstraints.HORIZONTAL, insets, 0, 0));
		JPanel modelpanel = new JPanel(new BorderLayout());
		modelpanel.add("Center", modelname);
		JButton modelButton = new JButton("Browse...")
		{
			public Dimension getMinimumSize()
			{
				return new Dimension((int)super.getMinimumSize().getWidth(), 0);
			}

			public Dimension getPreferredSize()
			{
				return new Dimension((int)super.getMinimumSize().getWidth(), 0);
			}
		};
		modelButton.setToolTipText(modeltooltip);
		modelpanel.add("East", modelButton);
		north.add(modelpanel, new GridBagConstraints(1, 0, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(configurationlabel, new GridBagConstraints(0, 1, 1, 1, 0, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(configchooser, new GridBagConstraints(1, 1, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));

		north.add(agentnamelabel, new GridBagConstraints(0, 2, 1, 1, 0, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(agentname, new GridBagConstraints(1, 2, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(classnamelabel, new GridBagConstraints(0, 3, 1, 1, 0, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(classname, new GridBagConstraints(1, 3, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(argumentslabel, new GridBagConstraints(0, 4, 1, 1, 0, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(arguments, new GridBagConstraints(1, 4, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(containerlabel, new GridBagConstraints(0, 5, 1, 1, 0, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
		north.add(container, new GridBagConstraints(1, 5, GridBagConstraints.REMAINDER, 1, 1, 0,
			GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));

		JPanel buts = new JPanel(new GridBagLayout());
		//buts.add(prodmode, new GridBagConstraints(0, 0, 1, 1, 1, 0,
		//	GridBagConstraints.WEST, GridBagConstraints.NONE, insets, 0, 0));
		buts.add(okbutton, new GridBagConstraints(0, 0, 1, 1, 1, 0,
			GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
		buts.add(cancelbutton, new GridBagConstraints(1, 0, 1, 1, 0, 0,
			GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
		buts.add(reloadbutton, new GridBagConstraints(2, 0, 1, 1, 0, 0,
			GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
		buts.add(resetbutton, new GridBagConstraints(3, 0, 1, 1, 0, 0,
			GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));
		buts.add(detailsbutton, new GridBagConstraints(4, 0, 1, 1, 0, 0,
			GridBagConstraints.EAST, GridBagConstraints.NONE, insets, 0, 0));

		n2.add("Center", north);
		n2.add("South", buts);

		// Set sizes
		int mw = (int)detailsbutton.getMinimumSize().getWidth();
		int pw = (int)detailsbutton.getPreferredSize().getWidth();
		okbutton.setMinimumSize(new Dimension(mw, (int)okbutton.getMinimumSize().getHeight()));
		okbutton.setPreferredSize(new Dimension(pw, (int)okbutton.getPreferredSize().getHeight()));
		cancelbutton.setMinimumSize(new Dimension(mw, (int)cancelbutton.getMinimumSize().getHeight()));
		cancelbutton.setPreferredSize(new Dimension(pw, (int)cancelbutton.getPreferredSize().getHeight()));
		reloadbutton.setMinimumSize(new Dimension(mw, (int)reloadbutton.getMinimumSize().getHeight()));
		reloadbutton.setPreferredSize(new Dimension(pw, (int)reloadbutton.getPreferredSize().getHeight()));
		resetbutton.setMinimumSize(new Dimension(mw, (int)resetbutton.getMinimumSize().getHeight()));
		resetbutton.setPreferredSize(new Dimension(pw, (int)resetbutton.getPreferredSize().getHeight()));

		// Add gui listeners.

		okbutton.addActionListener(this);
		cancelbutton.addActionListener(this);
		resetbutton.addActionListener(this);
		reloadbutton.addActionListener(this);
		detailsbutton.addActionListener(this);

		modelButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent ae)
			{
				if(modelchooser.showDialog(StartDialog.this, "load")==JFileChooser.APPROVE_OPTION)
				{
					File file = modelchooser.getSelectedFile();
					String	model	= file!=null ? ""+file : null;

					if(file!=null && file.getName().endsWith(".jar"))
					{
						// Start looking into the jar-file for description-files
						try
						{
							DynamicURLClassLoader.addURLToInstance(new URL("file", "", file.toString()));

							JarFile jarFile = new JarFile(file);
							Enumeration e = jarFile.entries();
							List	models	= new ArrayList();
							while (e.hasMoreElements()) 
							{
								ZipEntry jarFileEntry = (ZipEntry) e.nextElement();
								if(jarFileEntry.getName().endsWith(".agent.xml"))
								{
									models.add(jarFileEntry.getName());
								}
							}
							jarFile.close();

							if(models.size()>1)
							{
								Object[]	choices	= models.toArray(new String[models.size()]);
								JTreeDialog td = new JTreeDialog(
									(Frame)StartDialog.this.getParent(),
									"Select Agent Model", true,
									"Select an agent model to load:",
									(String[])choices, (String)choices[0]);
								td.setVisible(true);
								model = td.getResult();

								/*model = (String)JOptionPane.showInputDialog(
									StartDialog.this,
									"Select an agent model to load:",
									"Select Agent Model",
									JOptionPane.QUESTION_MESSAGE,
									null, choices, choices[0]);*/
							}
							else if(models.size()==1)
							{
								model	= (String)models.get(0);
							}
							else
							{
								model	= null;
							}
						}
						catch(Exception e)
						{
							//e.printStackTrace();
						}
					}

					//System.out.println("0");
					lastfile	= model;
					loadModel(model);
				}
			}
		});

		detailslistener	= new ChangeListener()
		{
			public void	stateChanged(ChangeEvent ce)
			{
				//System.out.println("SystemEvent: "+ce);
				Object	id	= modeldesc.getId(modeldesc.getSelectedComponent());
				if(id instanceof String && ((DefaultComboBoxModel)modelname
					.getModel()).getIndexOf(id)!=-1 && loadedmodels.get(id)!=null)
				{
					model = (IMElement)loadedmodels.get(id);
					lastfile	= ((IMCapability)model).getFilename();
					//System.out.println("Id: "+id);
					//loadModel((String)id);
					int bn = modelname.getActionListeners().length;
					modelname.removeActionListener(StartDialog.this);
					modelname.getModel().setSelectedItem(id);

					//System.out.println("Model loaded: "+model);
					if(model.getName()!=null)
					{
						agentname.setText(model.getName());
					}
//					if(model instanceof IAgentType)
//					{
//						IAgentType agent = (IAgentType)model;
//						if(agent.getClassName()!=null)
//						{
//							classname.setText(agent.getClassName());
//						}
//					}
					// todo: HACK!!!
					if(model instanceof IMBDIAgent)
					{
						IMBDIAgent agent = (IMBDIAgent)model;
						// Todo: support custom class name.
//						if(agent.hasClassname())
//						{
//							classname.setText(agent.getClassname());
//						}
//						else
						{
							// Use default adapter agent.
							classname.setText(JadeAgentAdapter.class.getName());
						}
					}

					int an = modelname.getActionListeners().length;
					if(bn == an+1)
						modelname.addActionListener(StartDialog.this);
					else
						System.out.println("WARNING, action listeners corrupted: "+bn+" "+an);
					
					// Adjust state of start button depending on model checking state.
					okbutton.setEnabled(!Configuration.getConfiguration().isModelChecking() || model.getReport().isEmpty());
				}
			}
		};
		modeldesc.addChangeListener(detailslistener);

		// try to load arguments from properties.
		try
		{
			props = new Properties();
			props.load(new FileInputStream("rma.properties"));
			//System.out.println("Found props: "+props);

			// Add the jars added to the classpath.
			String url = "";
			for(int i = 0; url!=null; i++)
			{
				url = props.getProperty("url_"+i);
				if(url!=null)
					DynamicURLClassLoader.addURLToInstance(new URL(url));
			}

			// Load the modelnames.
			String mo = "";
			for(int i = 0; mo!=null; i++)
			{
				mo = props.getProperty("model_"+i);
				if(mo!=null)
					modelname.addItem(mo);
			}
			// Hack!!! Has to be added after adding items to combo box,
			// but before calling loadModel()!
			modelname.addActionListener(this);

			if(props.getProperty("model")!=null)
			{
				lastfile	= props.getProperty("model");
				File selected = new File(lastfile);
				modelchooser.setCurrentDirectory(selected);
				modelchooser.setSelectedFile(selected);
				//System.out.println("1");
				loadModel(lastfile);
				if(props.getProperty("config")!=null)
					configchooser.setSelectedItem(props.getProperty("config"));
			}
			if(props.getProperty("arguments")!=null)
			{
				arguments.setText(props.getProperty("arguments"));
			}
			if(props.getProperty("name")!=null)
			{
				agentname.setText(props.getProperty("name"));
			}
			if(props.getProperty("class")!=null)
			{
				classname.setText(props.getProperty("class"));
			}
			if(props.getProperty("details")!=null)
			{
				detailson = new Boolean(props.getProperty("details")).booleanValue();
			}
		}
		catch(Exception e)
		{
			// Hack!!! if rma.properties couldn't be loaded, add listener now.
			modelname.addActionListener(this);
			//e.printStackTrace();
		}

		if(!detailson)
		{
			detailsbutton.setText("Description -->");
			this.pack();
		}
		else
		{
			detailsbutton.setText("Description <--");
			getContentPane().add("Center", modeldesc);
			this.setSize(800, 600);
		}
	}

	/**
	 * public Dimension getPreferredSize () {
	 * return (new Dimension(450,150));
	 * }
	 */

	public void actionPerformed(ActionEvent evt)
	{
		choice = CANCEL_BUTTON;
		if(evt.getSource()==okbutton)
		{
			choice = OK_BUTTON;
			// try to save arguments in properties.
			saveProperties();

			dispose();
		}
		else if(evt.getSource()==cancelbutton)
		{
			//System.out.println("Canceled");
			dispose();
		}
		else if(evt.getSource()==reloadbutton)
		{
			if(lastfile!=null)
			{
				SXML.clearModelCache(lastfile+".cam"); // todo: Hack!
				loadModel(lastfile);
			}
		}
		else if(evt.getSource()==resetbutton)
		{
			//System.out.println("2");
			modeldesc.removeAll();
			loadModel(null);
			agentname.setText("");
			classname.setText("");
			arguments.setText("");
			configchooser.removeAllItems();
			((DefaultComboBoxModel)modelname.getModel()).removeAllElements();
			props = new Properties();
			DynamicURLClassLoader.resetInstance();
			saveProperties();
		}
		else if(evt.getSource()==detailsbutton)
		{
			//System.out.println(detailson);
			if(!detailson)
			{
				smallsize	= getSize(smallsize);
				detailson = true;
				detailsbutton.setText("Description <--");
				getContentPane().add("Center", modeldesc);
				StartDialog.this.invalidate();
				StartDialog.this.doLayout();
				StartDialog.this.repaint();
				//StartDialog.this.pack();
				if(largesize==null)
				{
					largesize	= getSize();
					largesize.height	*= 2.5;
				}
				setSize(smallsize.width, largesize.height);
			}
			else
			{
				largesize	= getSize(largesize);
				detailson = false;
				detailsbutton.setText("Description -->");
				getContentPane().remove(modeldesc);
				StartDialog.this.invalidate();
				StartDialog.this.doLayout();
				StartDialog.this.repaint();
				if(smallsize==null)
					smallsize	= getPreferredSize();
				setSize(largesize.width, smallsize.height);
			}
		}
		else if(evt.getSource()==modelname)
		{
			//System.out.println("++++++++++++++"+evt.getActionCommand());
			lastfile	= (String)modelname.getSelectedItem();
			loadModel(lastfile);
		}
	}

	public Dimension getMinimumSize()
	{
		Dimension max = getMaximumSize();
		Dimension min = super.getMinimumSize();
		return new Dimension((int)Math.min(min.getWidth(), max.getWidth()),
				(int)Math.min(min.getHeight(), max.getHeight()));
	}

	public Dimension getPreferredSize()
	{
		Dimension max = getMaximumSize();
		Dimension pref = super.getPreferredSize();
		return new Dimension((int)Math.min(pref.getWidth(), max.getWidth()),
				(int)Math.min(pref.getHeight(), max.getHeight()));
	}

	public Dimension getMaximumSize()
	{
		GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment()
			.getDefaultScreenDevice();
		Rectangle r = device.getDefaultConfiguration().getBounds();
		return new Dimension((int)(r.getWidth()*0.75), (int)(r.getHeight()*0.75));
	}

	public void loadModel(String adf)
	{
		//System.out.println("loadModel: "+adf+" "+modelname.getActionListeners().length+" "+SUtil.arrayToString(modelname.getActionListeners()));

		int bn = modelname.getActionListeners().length;
		int bd = modeldesc.getChangeListeners().length;
		modelname.removeActionListener(this);
		modeldesc.removeChangeListener(detailslistener);
		int an = modelname.getActionListeners().length;
		int ad = modeldesc.getChangeListeners().length;

		try
		{
			// Hack ???

			//System.out.println("AL: "+modelname.getActionListeners().length);
			if(adf!=null && !adf.equals(""))
			{
				model = SXML.loadAgentModel(adf, null);
				loadedmodels.put(adf, model);

				if(((DefaultComboBoxModel)modelname.getModel()).getIndexOf(adf)==-1)
				{
					((DefaultComboBoxModel)modelname.getModel()).addElement(adf);
				}
				modelname.setSelectedItem(adf);
				//System.out.println("Model loaded: "+model);
				if(model.getName()!=null)
				{
					agentname.setText(model.getName());
				}
//				if(model instanceof IAgentType)
//				{
//					IAgentType agent = (IAgentType)model;
//					if(agent.getClassName()!=null)
//					{
//						classname.setText(agent.getClassName());
//					}
//				}
				configchooser.removeAllItems();
				if(model instanceof IMBDIAgent)
				{
					IMBDIAgent agent = (IMBDIAgent)model;
					// Todo: support custom class name.
//					if(agent.hasClassname())
//					{
//						classname.setText(agent.getClassname());
//					}
//					else
					{
						// Use default adapter agent.
						classname.setText(JadeAgentAdapter.class.getName());
					}
					// Add all known agent configuration names to the config chooser.
					IMConfiguration[]	states	= agent.getConfigurationbase().getConfigurations();
					for(int i=0; i<states.length; i++)
					{
						((DefaultComboBoxModel)configchooser.getModel()).addElement(states[i].getName());
					}
					IMConfiguration defstate = agent.getConfigurationbase().getDefaultConfiguration();
					if(defstate!=null)
						configchooser.getModel().setSelectedItem(defstate.getName());
				}
				/*if(props2.getProperty("arguments")!=null)
					arguments.setText(props2.getProperty("arguments"));
				else
					arguments.setText(profile.toString()); */

				//System.out.println("++++++++++++++++");
				//modeldesc.addElement(model.getEncodableRepresentation());
				String clazz	= SReflect.getInnerClassName(model.getClass());
				if(clazz.endsWith("Data"))
					clazz	= clazz.substring(0, clazz.length()-4);
				Icon	icon	= GuiProperties.getElementIcon(clazz);

				IReport	report	= Configuration.getConfiguration().isModelChecking()? model.getReport(): null;
           		if(report!=null && !report.isEmpty())
	   	            modeldesc.addHTMLContent(model.getName(), icon, report.toHTMLString(), adf, report.getDocuments());
	            else
	            	modeldesc.addHTMLContent(model.getName(), icon, getDescription(model), adf, null);
				//	model instanceof IMCapability ? ((IMCapability)model).getFullName() : model.getName());
				//System.out.println("----------------");

           		// Adjust state of start button depending on model checking state.
				okbutton.setEnabled(report==null || report.isEmpty());
			}
			else
			{
				model = null;
				modeldesc.addTextContent("Model", null, "No model loaded.", "model");
           		// Non-jadex agents can always be started
				okbutton.setEnabled(true);
			}
		}
		catch(Exception e)
		{
			//e.printStackTrace();
			model = null;
			StringWriter sw = new StringWriter();
			e.printStackTrace(new PrintWriter(sw));
			modeldesc.addTextContent("Error", null, "No model loaded:\n"+sw, "error");
			modelname.setSelectedItem("");
       		// Broken agents can never be started
			okbutton.setEnabled(false);

			if(adf!=null && !adf.equals("") && !detailson)
			{
				detailson = true;
				detailsbutton.setText("Description <--");
				getContentPane().add("Center", modeldesc);
				StartDialog.this.invalidate();
				StartDialog.this.doLayout();
				StartDialog.this.repaint();
				StartDialog.this.pack();
			}
		}

		if(bn == an+1)
			modelname.addActionListener(this);
		else
			System.out.println("WARNING, action listeners corrupted: "+bn+" "+an);

		if(bd == ad+1)
			modeldesc.addChangeListener(detailslistener);
		else
			System.out.println("WARNING, change listeners corrupted: "+bd+" "+ad);


/*		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				modeldesc.getVerticalScrollBar().setValue(0);
				modeldesc.getHorizontalScrollBar().setValue(0);
			}
		});
*/	}

	/**
	 * Save the properties.
	 */
	public void saveProperties()
	{
		try
		{
			URL[] urls = DynamicURLClassLoader.getURLsFromInstance();
			for(int i = 0; i<urls.length; i++)
			{
				//System.out.println("saving url: "+urls[i]);
				props.setProperty("url_"+i, ""+urls[i]);
			}
			for(int i = 0; i<modelname.getModel().getSize(); i++)
			{
				props.setProperty("model_"+i, (String)modelname.getModel().getElementAt(i));
			}
			String m = (String)modelname.getSelectedItem();
			if(m!=null) props.setProperty("model", m);
			String c = (String)configchooser.getSelectedItem();
			if(c!=null) props.setProperty("config", c);
			props.setProperty("name", agentname.getText());
			props.setProperty("class", classname.getText());
			props.setProperty("arguments", arguments.getText());
			props.setProperty("details", ""+detailson);
			OutputStream out = new FileOutputStream("rma.properties");
			props.store(out, "default agent startup arguments");
			out.close();
		}
		catch(Exception e)
		{
			System.out.println("Could not save properties. Reason:"+e);
			e.printStackTrace();
		}
	}

	//-------- static part --------

	protected static Map	dias	= new WeakHashMap();

	/**
	 *  Create a start dialog.
	 *  @param frame	The parent frame.
	 *  @param containername	The initial value for the container name.
	 *  @return The result object.
	 */
	protected static Result	showDialog(Frame frame, String containername)
	{
		// Get dialog (from cache if present).
		StartDialog	dia	= create(frame);
		if(dia.getLocation().getX()==-1)
		{
			//dia.pack();
			dia.setLocation(frame.getX()+(frame.getWidth()-dia.getWidth())/2,
				frame.getY()+(frame.getHeight()-dia.getHeight())/2);
		}
		dia.choice	= CANCEL_BUTTON;
		dia.container.setText(containername);
		dia.setVisible(true);

		Result	result	= new Result();
		result.choice	= dia.choice;
		result.modelname	= (String)dia.modelname.getSelectedItem();
		result.configname	= (String)dia.configchooser.getSelectedItem();
		result.agentname	= dia.agentname.getText();
		result.classname	= dia.classname.getText();
		result.arguments	= dia.arguments.getText();
		result.container	= dia.container.getText();

		return result;
	}

	/**
	 *  Speedup loading by pre-creating dialog.
	 */
	protected static StartDialog	create(Frame frame)
	{
		StartDialog	dia	= (StartDialog)dias.get(frame);
		if(dia==null)
		{
			dia	= new StartDialog(frame);
			dias.put(frame, dia);
			// Hack: Initialize location later.
			dia.setLocation(-1, -1);
		}
		return dia;
	}

	/**
	 *  Get the description for a model.
	 *  Only required because JBind doesn't allow access to XML comments.
	 */
	protected String	getDescription(IMElement model)
	{
		String	ret	= null;
		if(model instanceof IMCapability)
		{
			try
			{
				// Try to extract first comment from file.
				InputStream	is	= SUtil.getResource(((IMCapability)model).getFilename());
				int	read;
				while((read=is.read())!=-1)
				{
					if(read=='<')
					{
						read	= is.read();
						if(Character.isLetter((char)read))
						{
							// Found first tag, use whatever comment found up to now.
							break;
						}
						else if(read=='!' && is.read()=='-' && is.read()=='-')
						{
							// Found comment.
							StringBuffer	comment	= new StringBuffer();
							while((read=is.read())!=-1)
							{
								if(read=='-')
								{
									if((read=is.read())=='-')
									{
										if((read=is.read())=='>')
										{
											// Finished reading <!-- ... --> statement
											ret	= comment.toString();
											break;
										}
										else
										{
											comment.append("--");
											comment.append((char)read);
										}
									}
									else
									{
										comment.append('-');
										comment.append((char)read);
									}
								}
								else
								{
									comment.append((char)read);
								}
							}
						}
					}
				}
				is.close();
			}
			catch(Exception e)
			{
				ret	= "No description available: "+e;
			}
		}
		else
		{
			ret	= "No description available.";
		}
		return ret;
	}

	//-------- inner classes --------

	/**
	 *  Result of a start dialog invocation.
	 */
	public static class Result
	{
		public int	choice;
		public String	agentname;
		public String	classname;
		public String	arguments;
		public String	container;
		public String	modelname;
		public String	configname;
	}
}

